const PLAYER_FACADE_VERSION = '2.2';

/**
 * A player proxy class
 * (last change 2018-09-06T07:39:21.053Z)
 * @module FacadeBase
 * @class
 */
function FacadeBase() {

    /**
	 * Requests the player facade version (API version) to ensure the particular versions of `smartclientcore` and player facade
	 * are supported.
	 * @param {string} apiFacadeVersion - Required API version
	 * @returns {string} Supported API version
	 */
	this.handshakeVersion = function _(apiFacadeVersion) {
		return PLAYER_FACADE_VERSION;
    };

	/**
	 * Requests the video player to load and play the specified media file.
	 * @param {string} url - Media file URI
	 * @param {Boolean} autoPlay - Whether the requested media file starts playback immediately.
	 * @param {Boolean} disableSeeking - Whether the requested source can be seeked back and forth.
	 * @returns {Promise} Resolves if the player accepted the media file request
	 */
	this.load = function _(url, autoPlay = true, disableSeeking = true) {
        Android.onLoadMedia(url, autoPlay, disableSeeking);
        return Promise.resolve(true);
    };

	/**
	 * Sets up a function that will be called whenever the specified event is delivered to the video player.
	 * @param {string} type - A case-sensitive string representing the player event type to listen for.
	 * @param {Function} callback - The function which receives a notification when an event of the specified type occurs.
	 */
	this.addEventListener = function _(type, callback) {
        let eventDispatcher = getEventDispatcher();
        eventDispatcher.addEventListener(type, callback);
    };

	/**
	 * Removes an event listener from the video player that has been registered with `addEventListener()`
	 * _*Note:* The event listener to be removed must match with the listener that has been added._
	 * @param {string} type - A case-sensitive string representing the player event type to listen for.
	 * @param {Function} callback - The function which receives a notification when an event of the specified type occurs.
	 */
	this.removeEventListener = function _(type, callback) {
        let eventDispatcher = getEventDispatcher();
        eventDispatcher.removeEventListener(type, callback);
    };

	/**
	 * Requests the video player to lock the current content.
	 * _This happens before starting a new ad break_
	 * @returns {SourceObject} Current content source and playhead progress
	 */
	this.setCurrentContentSource = function _() {
        return {
            progressive: null
        }
    };

	/**
	 * @returns {SourceObject} Current content source and playhead progress
	 */
	this.getCurrentContentSource = function _() {
        return {
            progressive: null
        }
    };

	/**
	 * Requests the video player to return to the content and last known playhead position.
	 * @returns {Promise} Resolves if the player accepted the request
	 */
	this.resetCurrentContentSource = function _() {
        return Promise.resolve(true);
    };

    /**
     * Requests the video player to return to the content and last known playhead position.
     * @returns {Promise} Resolves if the player accepted the request
     */
    this.releaseContent = function _() {
        return Promise.resolve(true);
    };

    /**
     * Requests the video player to return to the content and last known playhead position.
     * @returns {Promise} Resolves if the player accepted the request
     */
    this.lockContent = function _() {
        return Promise.resolve(true);
    };

	/**
	 * Requests details about the most recent error that has occurred on the video player.
	 * @returns {MediaError} Description object of the most recent error or `null`.
	 */
	this.getError = function _() {
	    let error = Android.getError();
	    console.log("getError() " + error);
	    return error;
	};

	/**
	 * Requests the current playhead position from the video player.
	 * @returns {Number} Current time in seconds.
	 */
	this.getCurrentTime = function _() {
	    return Android.onGetCurrentMediaPosition();
    };

	/**
	 * Requests the content-duration from the video player.
	 * @returns {Number} Total time in seconds.
	 */
	this.getDuration = function _() {
	    return Android.onGetCurrentMediaDuration();
    };

	this.getVideoElement = function _() {
	    console.log("getVideoElement()");
    };

	/**
	 * Requests the video player to change the volume.
	 * @param {Number} volume - Requested volume level between 0-1
	 * @returns {Promise} Resolves if the video player accepted the volume change request
	 */
	this.setVolume = function _(volume) {
        Android.onSetVolume(volume);
        return Promise.resolve(true);
    };

	/**
	 * Requests the current volume level from the video player.
	 * @returns {Number} The current volume level between 0-1
	 */
	this.getVolume = function _() {
	    return Android.onGetVolume();
    };

	/**
	 * Requests the current muted state from the video player.
	 * @returns {Boolean} Muted state
	 */
	this.isMuted = function _() {
	    return Android.onIsMuted();
    };

	/**
	 * Requests the video player to mute playback.
	 * @returns {Promise} Resolves if the video player has been muted
	 */
	this.mute = function _() {
        Android.onSetMute(true);
        return Promise.resolve(true);
    };

	/**
	 * Requests the video player to unmute playback.
	 * @returns {Promise} Resolves if the video player has been unmuted
	 */
	this.unmute = function _() {
        Android.onSetMute(false);
        return Promise.resolve(true);
    };

	/**
	 * Requests the current play state from the video player.
	 * @returns {Boolean} Playback state
	 */
	this.isPaused = function _() {
	    return Android.onIsPaused();
    };

	/**
	 * Requests the video player to pause playback.
	 * @returns {Promise} Resolves if the video player has been paused
	 */
	this.pause = function _() {
        Android.onPause();
        return Promise.resolve(true);
    };

	/**
	 * Requests the video player to start (resume) playback.
	 * @returns {Promise} Resolves if the video player has been resumed
	 */
	this.play = function _() {
        Android.onPlay();
        return Promise.resolve(true);
    };

	/**
	 * Requests the video player to change the playhead position to the specified time.
	 * @param {Number} offset - Requested playhead position
	 * @returns {Promise} Resolves if the video player has finished seeking to the specified position
	 */
	this.seek = function _(offset) {
        Android.onSeek(offset);
        return Promise.resolve(true);
    };
}
